#include "ODEGelenk.cpp"
#include <ode/ode.h>             
#include <drawstuff/drawstuff.h> 
#include <string>
#include <iostream> 

class NAO{
	public:
		void baueRoboter(dWorldID welt){
			double y=0;
			double xRechts=2,xLinks=0; //Die xKoordinate vom rechten Bein
			double hHand=1, hVorderarm=2, hEllenbogen=3.25, hBizeps=4.5;
			double lHand=0.5, lVorderarm=1, lEllenbogen=0.25, lBizeps=1;

	
			DSRHand = new DSKoerperteil(welt, xRechts, y, hHand, lHand); //Hier wird festgelegt wo die Körpterteile Anfangs ihren Mittelpunkt haben sollen
			DSREllenbogen = new DSKoerperteil(welt, xRechts, y, hEllenbogen, lEllenbogen);	
			DSRVorderarm= new DSKoerperteil(welt, xRechts, y, hVorderarm, lVorderarm);
			DSRBizeps= new DSKoerperteil(welt, xRechts, y, hBizeps, lBizeps);
			DSRBizeps->festSetzen(); // Der Bizeps wird "fixiert" damit die Bewegungen besser nachvollziebar sind. 

			DSLHand = new DSKoerperteil(welt, xLinks, y, hHand, lHand);
			DSLEllenbogen = new DSKoerperteil(welt, xLinks, y, hEllenbogen, lEllenbogen);
			DSLVorderarm= new DSKoerperteil(welt, xLinks, y, hVorderarm, lVorderarm);
			DSLBizeps= new DSKoerperteil(welt, xLinks, y, hBizeps, lBizeps);
			DSLBizeps->festSetzen();	


			//Die Koerperteile werden hier mit den Entsprechenden Gelenken verknüpft und die Bewegungsachse der Gelenke festgelegt
			ODERWristYaw = new ODEGelenk(DSRHand, DSRVorderarm, welt, xRechts, y, 1.25, 0, 0, -1);
			ODEREllbowRoll = new ODEGelenk(DSRVorderarm, DSREllenbogen, welt, xRechts, y, 3, 0, -1, 0);
			ODEREllbowYaw = new ODEGelenk(DSREllenbogen, DSRBizeps, welt, xRechts, y, 3.50, 0, 0, -1);
	
			ODELWristYaw = new ODEGelenk(DSLHand, DSLVorderarm, welt, xLinks, y, 1.5, 0, 0, 1);
			ODELEllbowRoll = new ODEGelenk(DSLVorderarm, DSLEllenbogen, welt, xLinks, y, 3, 0, 1, 0);
			ODELEllbowYaw = new ODEGelenk(DSLEllenbogen, DSLBizeps, welt, xLinks, y, 3.50, 0, 0, 1);

 			//Entsprechend der NAO-Spezifikation werden hier die Minmalen und Maximalen Winkel festegelegt
			ODERWristYaw->setMinMax(-1.8238,1.8238);
			ODEREllbowRoll->setMinMax(-1.5446,-0.0349);
			ODEREllbowYaw->setMinMax(-2.0857,2.0857);

			ODELWristYaw->setMinMax(-1.8238,1.8238);
			ODELEllbowRoll->setMinMax(-1.5446,-0.0349);
			ODELEllbowYaw->setMinMax(-2.0857,2.0857);
			
			//Aus aesthetischen Gruenden wird jedes Gelenk leicht ausgelenkt
			ODERWristYaw->setWinkel(-0.1);
			ODELWristYaw->setWinkel(-0.1);

			ODEREllbowRoll->setWinkel(-0.1);
			ODELEllbowRoll->setWinkel(-0.1);
			
			ODEREllbowYaw->setWinkel(-0.1);
			ODELEllbowYaw->setWinkel(-0.1);
		}

		double getAngles(string gelenk) { //Winkel in Grad
			
			//intern arbeitet die Winkelklasse mit RAD anstatt DEG, weswegen hier eine Umrechnung stattfinden muss
			//Fuer jeden string wird ueberprueft welches Gelenk gemeint ist und auf dem entsprechenden Objekt wird dann die getWinkel aufgerufen

		    if(gelenk.compare("RWristYaw")==0){
				return (ODERWristYaw->getWinkel()*180/M_PI);
		    }
		    if(gelenk.compare("LWristYaw")==0){
				return ODELWristYaw->getWinkel()*180/M_PI;
		    }
		    if(gelenk.compare("REllbowRoll")==0){
		    	return ODEREllbowRoll->getWinkel()*180/M_PI;
		    }
		    if(gelenk.compare("LEllbowRoll")==0){
				return ODELEllbowRoll->getWinkel()*180/M_PI ;
		    }
		    if(gelenk.compare("REllbowYaw")==0){
				return ODEREllbowYaw->getWinkel()*180/M_PI ;
		    }
		    if(gelenk.compare("LEllbowYaw")==0){
				return ODELEllbowYaw->getWinkel()*180/M_PI ;
		    }
		}

		void setAngles(string gelenk, double winkel){ 
			//die Methode setAngles ist analog zu getAngels nur, dass Werte gesetzt werden
    		if(gelenk.compare("RWristYaw")==0){
        		ODERWristYaw->setWinkel(winkel*M_PI/180);
    		}
			if(gelenk.compare("LWristYaw") == 0){
				  ODELWristYaw->setWinkel(winkel*M_PI/180);
			}
			 if(gelenk.compare("REllbowRoll")==0){
				  ODEREllbowRoll->setWinkel(winkel*M_PI/180);
			 }
			 if(gelenk.compare("LEllbowRoll")==0){
				  ODELEllbowRoll->setWinkel(winkel*M_PI/180);
			 }
			 if(gelenk.compare("REllbowYaw")==0){
				  ODEREllbowYaw->setWinkel(winkel*M_PI/180);
			 }
			 if(gelenk.compare("LEllbowYaw")==0){
				  ODELEllbowYaw->setWinkel(winkel*M_PI/180);
			 }
		}
		
		void openHand(string hand) {

			//auch die Methoden openHand und closeHand sind sehr aehnlich zu get und setAngels
			if(hand.compare("RHand")==0){
				DSRHand->oeffnen();
			 }
			if(hand.compare("LHand")==0){
				DSLHand->oeffnen();
			 }
		}
		void closeHand(string hand) {
			if(hand.compare("RHand")==0){
				DSRHand->schliessen();
			 }
			if(hand.compare("LHand")==0){
				DSLHand->schliessen();
			 }
		}

		int getHand(string hand) { //1=offen, 0 = geschlossen

			//diese Methode wird zwar nicht fuer die NAO Spezifikationen benoetigt, aber fuer automatische Tests ist sie sinnvoll
			if(hand.compare("RHand")==0){
				return DSRHand->getHand();
			 }
			if(hand.compare("LHand")==0){
				return DSLHand->getHand();
			 }
		}
	
		void zeichneRoboter(){
			//der Aufruf wird an alle Koerperteile einfach weitergegeben
			DSRBizeps->zeichneKoerperteil();
			DSRVorderarm->zeichneKoerperteil();
			DSREllenbogen->zeichneKoerperteil();
			DSRHand->zeichneKoerperteil();

			DSLBizeps->zeichneKoerperteil();
			DSLVorderarm->zeichneKoerperteil();
			DSLEllenbogen->zeichneKoerperteil();
			DSLHand->zeichneKoerperteil();
		}

		void bewegeRoboter(){
			//der Aufruf wird an alle Gelenke einfach weitergegeben
			ODERWristYaw->bewege();
			ODEREllbowRoll->bewege();
			ODEREllbowYaw->bewege();
			
			ODELWristYaw->bewege();
			ODELEllbowRoll->bewege();
			ODELEllbowYaw->bewege();
		}

		void zustandAusgeben(){
			//zustandAusgeben() sammelt alle Werte aller Gelenke und gibt sie gesammelt als Tabelle aus
			cout  <<
		"\n RWristYaw "<< (ODERWristYaw->getWinkel())*180/M_PI <<
		"\n LWristYaw " << (ODELWristYaw->getWinkel())*180/M_PI  << 
		"\n REllbowRoll "<<(ODEREllbowRoll->getWinkel())*180/M_PI  <<
		"\n LEllbowRoll "<<(ODELEllbowRoll->getWinkel())*180/M_PI  <<
		"\n REllbowYaw " <<(ODEREllbowYaw->getWinkel())*180/M_PI  <<
		"\n LEllbowYaw " <<(ODELEllbowYaw->getWinkel())*180/M_PI  << endl;
		}

		void zerstoereRoboter(){
			//alle Objekte von NAO (Gelenke und Koerperteile) werden hier geschloescht
			delete DSRBizeps,DSRVorderarm, DSRHand, DSREllenbogen;
			delete ODEREllbowYaw, ODEREllbowRoll, ODERWristYaw;
			delete DSLBizeps,DSLVorderarm, DSLHand, DSLEllenbogen;
			delete ODELEllbowYaw, ODELEllbowRoll, ODELWristYaw;
		}
	private:
		DSKoerperteil *DSRBizeps,*DSRVorderarm, *DSRHand, *DSREllenbogen;
		DSKoerperteil *DSLBizeps,*DSLVorderarm, *DSLHand, *DSLEllenbogen;

		ODEGelenk *ODEREllbowYaw, *ODEREllbowRoll, *ODERWristYaw;
		ODEGelenk *ODELEllbowYaw, *ODELEllbowRoll, *ODELWristYaw;

};
