
#include <ode/ode.h>             
#include <drawstuff/drawstuff.h> 
#include <string>
#include <iostream> 
#include <sys/time.h>

#include "NAO.cpp"



using namespace std;

class simulation{
	public:  

		simulation(int argc, char **argv){
			this->argc = argc;
			this->argv = argv;
			this->zeit=0;

			gettimeofday(&startpunkt,0);
			//Initialisieren von Variablen, die für DrawStuff wichtig sind
			 fn.version = DS_VERSION;
			 fn.start   = &start;
			 fn.step    = &simLoop;
			 fn.command = &command;
			 fn.path_to_textures = "textures";

			 dInitODE();  // Intialisiere ODE
			 welt = dWorldCreate();  //Erstellen der Welt

			 dMass mass;
			roboter = new NAO();
			roboter->baueRoboter(welt);
		}
	
		static void start()  
		 
		{
			 float xyz[3] ={  4, 4, 3};    // Postition der Kamera 
			 float hpr[3] = { -120.0, 4.50, 0.00};  // Richtung der Kamera 
			 dsSetViewpoint(xyz,hpr);               // initialisiert
		}



		static void command(int cmd)  //Hier werden die Tastatureingaben verarbeitet
		{
			if(tast==1) return;
			if(cmd=='z' || cmd=='Z') roboter->zustandAusgeben();

			 switch (cmd)
			 {
			 case 'f': 
					roboter->setAngles("RWristYaw",roboter->getAngles("RWristYaw")+1);			
				  break;
			 case 'F': 
					roboter->setAngles("RWristYaw",roboter->getAngles("RWristYaw")-1);			
				  break;
			 case 'j': 
					roboter->setAngles("LWristYaw",roboter->getAngles("LWristYaw")+1); 
				  break;
			 case 'J':
					roboter->setAngles("LWristYaw",roboter->getAngles("LWristYaw")-1); 
				  break;
			 case 'd':
					roboter->setAngles("REllbowRoll",roboter->getAngles("REllbowRoll")+1);
				  break;
			 case 'D':
					roboter->setAngles("REllbowRoll",roboter->getAngles("REllbowRoll")-1);
				  break;
			 case 'k': 
					roboter->setAngles("LEllbowRoll",roboter->getAngles("LEllbowRoll")+1);
				  break;
			 case 'K':
					roboter->setAngles("LEllbowRoll",roboter->getAngles("LEllbowRoll")-1);
				  break;
			 case 's': 
					roboter->setAngles("REllbowYaw",roboter->getAngles("REllbowYaw")+1);
				  break;
			 case 'S':
					roboter->setAngles("REllbowYaw",roboter->getAngles("REllbowYaw")-1);;
				  break;
			 case 'l': 
					roboter->setAngles("LEllbowYaw",roboter->getAngles("LEllbowYaw")+1);
				  break;
			 case 'L':
					roboter->setAngles("LEllbowYaw",roboter->getAngles("LEllbowYaw")-1);
				  break;
			 case 'g':
					roboter->openHand("RHand");
					break;
			 case 'G':
					roboter->closeHand("RHand");
					break;
			 case 'h':
					roboter->openHand("LHand");
					break;
			 case 'H':
					roboter->closeHand("LHand");
					break;
			 }
		}


		static void simLoop(int pause)
		{
			 dWorldStep(welt, 0.02);

			 dsSetColor(1.0,1.0,1.0); //zeichnen aller Koerperteilen

			roboter->zeichneRoboter();

			roboter->bewegeRoboter(); //Hier werden die Koerperteile (falls noetig) ein Stueck bewegt, sodass die Illusion einer Bewegung auftritt
			
			gettimeofday(&endpunkt,0);

			if(zeit!=0 && endpunkt.tv_sec-startpunkt.tv_sec>=zeit) {
				//wichtig falls nicht durchgehend simuliert werden soll
				dsStop();
			}	
		}

		//Die folgenden Methoden werden fast unveraendert an den Roboter weitergeben
		void emitState(){
			roboter->zustandAusgeben();	
		}
		void setAngles(string gelenk, double winkel){ //Winkel in Grad
			roboter->setAngles(gelenk, winkel);
		}
		double getAngles(string gelenk){ //Winkel in Grad
			return roboter->getAngles(gelenk);
		}
		void openHand(string hand){
			roboter->openHand(hand);
		}
		void closeHand(string hand){
			roboter->closeHand(hand);
		}
		int getHand(string hand){
			return roboter->getHand(hand);
		}



		int simulate(double zeit){
			simulation::zeit=zeit;
			gettimeofday(&startpunkt,0);

			dsSimulationLoop(this->argc, this->argv, 640, 570, &fn); 
		}

		int beendeSimulation(){
			roboter->zerstoereRoboter();
			dCloseODE(); 
			return 0;
		}

			
	private:
		static dWorldID welt;      
		int argc;
		char **argv;
		dsFunctions fn;
		static double zeit;
		static timeval startpunkt,endpunkt;

		static int tast; // eine Variable ob die simulation die Tastatureingaben ignorieren soll


		dJointID    fest;

		static NAO  *roboter;
};

